use bracket_lib::prelude::*;
use hecs::World;

use crate::resources::{flashing_message::FlashingMessageIntent, *};

mod player;

pub fn handle_intent(bterm: &mut BTerm, _world: &mut World, resources: &mut Resources) {
    if let Some(key) = bterm.key {
        if let Some(flashing_message) = &resources.flashing_message {
            if let Some(intent) = &flashing_message.intent {
                match intent {
                    FlashingMessageIntent::Start => {
                        if resources.sound_output.is_none() || resources.sound_effects.is_none() {
                            let sound_output = SoundOutput::new();
                            resources.sound_effects = Some(SoundEffects::new(&sound_output));
                            resources.sound_output = Some(sound_output);
                        }
                    }
                    FlashingMessageIntent::Quit => {
                        if key == VirtualKeyCode::Y {
                            bterm.quit()
                        }
                    }
                    FlashingMessageIntent::Save => todo!(),
                    FlashingMessageIntent::Restore => todo!(),
                    FlashingMessageIntent::Restart => todo!(),
                    &FlashingMessageIntent::NextLevel => {
                        resources.should_advance_level = true;
                    }
                }
            }
        }
    }
}

pub fn handle(world: &mut World, resources: &mut Resources, bterm: &mut BTerm) {
    if resources.flashing_message.is_some() && bterm.key.is_some() {
        handle_intent(bterm, world, resources);
        resources.flashing_message = None;
    }

    match bterm.key {
        None => {}
        Some(key) => match key {
            VirtualKeyCode::Left | VirtualKeyCode::J | VirtualKeyCode::Numpad4 => {
                player::try_move(-1, 0, world, resources);
            }
            VirtualKeyCode::U | VirtualKeyCode::Home | VirtualKeyCode::Numpad7 => {
                player::try_move(-1, -1, world, resources)
            }
            VirtualKeyCode::Up | VirtualKeyCode::I | VirtualKeyCode::Numpad8 => {
                player::try_move(0, -1, world, resources);
            }
            VirtualKeyCode::O | VirtualKeyCode::PageUp | VirtualKeyCode::Numpad9 => {
                player::try_move(1, -1, world, resources)
            }
            VirtualKeyCode::Right | VirtualKeyCode::K | VirtualKeyCode::Numpad6 => {
                player::try_move(1, 0, world, resources);
            }
            VirtualKeyCode::Comma | VirtualKeyCode::PageDown | VirtualKeyCode::Numpad3 => {
                player::try_move(1, 1, world, resources)
            }
            VirtualKeyCode::Down | VirtualKeyCode::M | VirtualKeyCode::Numpad2 => {
                player::try_move(0, 1, world, resources);
            }
            VirtualKeyCode::N | VirtualKeyCode::End | VirtualKeyCode::Numpad1 => {
                player::try_move(-1, 1, world, resources)
            }
            VirtualKeyCode::W => {
                player::whip(world, resources);
            }
            VirtualKeyCode::D => {
                resources.show_debug_info = !resources.show_debug_info;
            }
            VirtualKeyCode::Escape | VirtualKeyCode::Q => {
                if cfg!(not(target_family = "wasm")) {
                    resources.flashing_message = Some(FlashingMessage::new(
                        " Are you sure you want to quit (Y/N)? ",
                        Some(FlashingMessageIntent::Quit),
                    ));
                } else if let (Some(sound_effects), Some(sound_output)) =
                    (&mut resources.sound_effects, &mut resources.sound_output)
                {
                    sound_output.play_sound(sound_effects.bad_key.clone());
                }
            }
            VirtualKeyCode::P => {
                if let (Some(sound_effects), Some(sound_output)) =
                    (&mut resources.sound_effects, &mut resources.sound_output)
                {
                    sound_output.play_sound(sound_effects.pause.clone());
                }
                resources.flashing_message =
                    Some(FlashingMessage::from(" Press any key to resume game. "));
            }
            _ => {
                if let (Some(sound_effects), Some(sound_output)) =
                    (&mut resources.sound_effects, &mut resources.sound_output)
                {
                    sound_output.play_sound(sound_effects.bad_key.clone());
                }
            }
        },
    }
}
